
/* Ocamlyacc parser for Racontr */

%{
open Racontrast
%}

%token SEMI LPAREN RPAREN LBRACE RBRACE COMMA PLUS MINUS TIMES DIVIDE ASSIGN
%token NOT EQ NEQ LT LEQ GT GEQ AND OR IN 
%token RETURN IF ELSE FOR WHILE
%token EXTENDS SCENE ITEM CHARACTER MISSION ENDING CLASS VAR COLON
%token DEF ARROW
%token <int> LITERAL
%token <bool> BLIT
%token <string> ID FLIT
%token EOF

%start program
%type <Racontrast.program> program

%nonassoc NOELSE
%nonassoc ELSE
%right ASSIGN
%left OR
%left AND
%left EQ NEQ
%left LT GT LEQ GEQ
%left PLUS MINUS
%left TIMES DIVIDE
%right NOT

%% 

program:
    stmt program {$1 :: $2}
  | EOF {[]}

stmt:
    class_decl {$1}
  | var_decl {$1}
  | IF exp stmt %prec NOELSE {If ($2, $3, Block [])}
  | IF exp stmt ELSE stmt {If ($2, $3, $5)}
  | WHILE exp stmt {While ($2, $3)}
  | exp {ExpStmt $1}
  | block {$1}
  | ID ASSIGN exp { Assign ($1, $3) }
  | func {$1}

class_decl:
    CLASS ID LBRACE var_decl_list RBRACE {Class ($2, "Root", $4)}
  | CLASS ID EXTENDS ID LBRACE var_decl_list RBRACE {Class ($2, $4, $6)}

var_decl_list:
    var_decl var_decl_list {$1 :: $2}
  | {[]}

var_decl:
    VAR ID COLON ID ASSIGN exp { Var ($2, $6, $4) }

block:
    LBRACE stmt_list RBRACE {Block $2}
  | LBRACE RBRACE {Block []}

stmt_list:
    stmt stmt_list {$1 :: $2}
  | stmt {[$1]}

exp: 
  | LITERAL          { Literal($1)            }
  | FLIT	     { Fliteral($1)           }
  | BLIT             { BoolLit($1)            }
  | ID               { Id($1)                 }
  | ID LPAREN args RPAREN {Call ($1, $3)}
  | RETURN exp {Return $2}

  | exp PLUS   exp { Binop($1, "+",   $3)   }
  | exp MINUS  exp { Binop($1, "-",   $3)   }
  | exp TIMES  exp { Binop($1, "*",  $3)   }
  | exp DIVIDE exp { Binop($1, "/",   $3)   }
  | exp EQ     exp { Binop($1, "==", $3)   }
  | exp NEQ    exp { Binop($1, "!=",   $3)   }
  | exp LT     exp { Binop($1, "<",  $3)   }
  | exp LEQ    exp { Binop($1, "<=",   $3)   }
  | exp GT     exp { Binop($1, ">", $3) }
  | exp GEQ    exp { Binop($1, ">=",   $3)   }
  | exp AND    exp { Binop($1, "&&",   $3)   }
  | exp OR     exp { Binop($1, "||",    $3)   }
  | MINUS exp %prec NOT { Unop("-", $2)      }
  | NOT exp         { Unop("!", $2)          }

args:
    arg COMMA args { $1 :: $3 }
  | arg {[$1]}
  | {[]}

arg:
  exp {$1}

func:
  DEF ID LPAREN formal_args RPAREN ARROW ID stmt {Function ($2, $4, $7, $8)}

formal_args:
    formal_arg COMMA formal_args { $1 :: $3 }
  | formal_arg {[$1]}
  | {[]}

formal_arg:
    ID COLON ID {($1, $3)}